@@ -1,25 +1,75 @@
-from enum import IntEnum, Enum, unique
-from collections import namedtuple
+# This file is part of Gajim.
+#
+# Gajim is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published
+# by the Free Software Foundation; version 3 only.
+#
+# Gajim is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+
+from __future__ import annotations
+
+from typing import Any
+from typing import Union
+from typing import NamedTuple
+
+from enum import IntEnum
+from enum import Enum
+from enum import unique
+from functools import total_ordering
 
 from gi.repository import Gio
 
 from nbxmpp.namespaces import Namespace
+from nbxmpp.const import PresenceShow
+from nbxmpp.protocol import JID
 
 from gajim.common.i18n import _
 from gajim.common.i18n import Q_
 
-EncryptionData = namedtuple('EncryptionData', 'additional_data')
-EncryptionData.__new__.__defaults__ = (None,)  # type: ignore
 
-Entity = namedtuple('Entity', 'jid node hash method')
+STOP_EVENT = True
+PROPAGATE_EVENT = False
+
+MAX_MESSAGE_CORRECTION_DELAY = 300
+
+
+class EncryptionData(NamedTuple):
+    additional_data: Any = None
+
+
+class Entity(NamedTuple):
+    jid: JID
+    node: str
+    hash: str
+    method: str
+
+
+class RowHeaderType(IntEnum):
+    CONVERSATIONS = 0
+    PINNED = 1
 
 
 class AvatarSize(IntEnum):
     TAB = 16
+    SHOW_CIRCLE = 24
     ROSTER = 32
+    VCARD_HEADER = 40
+    ACCOUNT_SIDE_BAR = 40
+    WORKSPACE = 40
+    WORKSPACE_EDIT = 100
     CHAT = 48
     NOTIFICATION = 48
-    TOOLTIP = 125
+    CALL = 100
+    CALL_BIG = 200
+    GROUP_INFO = 100
+    TOOLTIP = 100
+    ACCOUNT_PAGE = 150
     VCARD = 200
     PUBLISH = 200
 
@@ -53,21 +103,15 @@ class KindConstant(IntEnum):
     SINGLE_MSG_SENT = 5
     CHAT_MSG_SENT = 6
     ERROR = 7
+    FILE_TRANSFER_INCOMING = 8
+    FILE_TRANSFER_OUTGOING = 9
+    CALL_INCOMING = 10
+    CALL_OUTGOING = 11
 
     def __str__(self):
         return str(self.value)
 
 
-@unique
-class ShowConstant(IntEnum):
-    ONLINE = 0
-    CHAT = 1
-    AWAY = 2
-    XA = 3
-    DND = 4
-    OFFLINE = 5
-
-
 @unique
 class TypeConstant(IntEnum):
     AIM = 0
@@ -100,25 +144,30 @@ class JIDConstant(IntEnum):
     NORMAL_TYPE = 0
     ROOM_TYPE = 1
 
+
 @unique
 class StyleAttr(Enum):
     COLOR = 'color'
     BACKGROUND = 'background'
     FONT = 'font'
 
+
 @unique
 class CSSPriority(IntEnum):
+    PRE_APPLICATION = 599
     APPLICATION = 600
     APPLICATION_DARK = 601
     DEFAULT_THEME = 610
     DEFAULT_THEME_DARK = 611
     USER_THEME = 650
 
+
 @unique
 class ButtonAction(Enum):
     DESTRUCTIVE = 'destructive-action'
     SUGGESTED = 'suggested-action'
 
+
 @unique
 class IdleState(Enum):
     UNKNOWN = 'unknown'
@@ -127,39 +176,8 @@ class IdleState(Enum):
     AWAKE = 'online'
 
 
-@unique
-class RequestAvatar(IntEnum):
-    SELF = 0
-    ROOM = 1
-    USER = 2
-
-
-@unique
-class PEPEventType(IntEnum):
-    ABSTRACT = 0
-    ACTIVITY = 1
-    TUNE = 2
-    MOOD = 3
-    LOCATION = 4
-    NICKNAME = 5
-    AVATAR = 6
-    ATOM = 7
-    BOOKMARKS = 8
-
-
-@unique
-class Chatstate(IntEnum):
-    COMPOSING = 0
-    PAUSED = 1
-    ACTIVE = 2
-    INACTIVE = 3
-    GONE = 4
-
-    def __str__(self):
-        return self.name.lower()
-
-
 class SyncThreshold(IntEnum):
+    NO_SYNC = -1
     NO_THRESHOLD = 0
 
     def __str__(self):
@@ -184,6 +202,11 @@ class Trust(IntEnum):
     VERIFIED = 3
 
 
+class Direction(IntEnum):
+    NEXT = 0
+    PREV = 1
+
+
 class Display(Enum):
     X11 = 'X11Display'
     WAYLAND = 'GdkWaylandDisplay'
@@ -199,6 +222,7 @@ class URIType(Enum):
     WEB = 'web'
     FILE = 'file'
     AT = 'at'
+    TEL = 'tel'
 
 
 class URIAction(Enum):
@@ -214,6 +238,7 @@ class MUCJoinedState(Enum):
     CREATING = 'creating'
     CAPTCHA_REQUEST = 'captcha in progress'
     CAPTCHA_FAILED = 'captcha failed'
+    PASSWORD_REQUEST = 'password request'
 
     def __str__(self):
         return self.name
@@ -242,6 +267,10 @@ class MUCJoinedState(Enum):
     def is_captcha_failed(self):
         return self == MUCJoinedState.CAPTCHA_FAILED
 
+    @property
+    def is_password_request(self):
+        return self == MUCJoinedState.PASSWORD_REQUEST
+
 
 class ClientState(IntEnum):
     DISCONNECTING = 0
@@ -276,6 +305,45 @@ class ClientState(IntEnum):
         return self == ClientState.AVAILABLE
 
 
+class SimpleClientState(Enum):
+    CONNECTING = 'connecting'
+    DISCONNECTED = 'disconnected'
+    CONNECTED = 'connected'
+    RESUME_IN_PROGRESS = 'resume-in-progress'
+
+    @property
+    def is_connecting(self):
+        return self == SimpleClientState.CONNECTING
+
+    @property
+    def is_connected(self):
+        return self == SimpleClientState.CONNECTED
+
+    @property
+    def is_disconnected(self):
+        return self == SimpleClientState.DISCONNECTED
+
+    @property
+    def is_resume_in_progress(self):
+        return self == SimpleClientState.RESUME_IN_PROGRESS
+
+
+class JingleState(Enum):
+    NULL = 'stop'
+    CONNECTING = 'connecting'
+    CONNECTION_RECEIVED = 'connection_received'
+    CONNECTED = 'connected'
+    ERROR = 'error'
+
+    def __str__(self):
+        return self.value
+
+
+class CallType(Enum):
+    AUDIO = 'audio'
+    VIDEO = 'video'
+
+
 MUC_CREATION_EXAMPLES = [
     (Q_('?Group chat name:Team'),
      Q_('?Group chat description:Project discussion'),
@@ -299,7 +367,8 @@ MUC_DISCO_ERRORS = {
     'remote-server-not-found': _('Remote server not found'),
     'remote-server-timeout': _('Remote server timeout'),
     'service-unavailable': _('Address does not belong to a group chat server'),
-    'subscription-required': _('Address does not belong to a group chat server'),
+    'subscription-required': _(
+        'Address does not belong to a group chat server'),
     'not-muc-service': _('Address does not belong to a group chat server'),
     'already-exists': _('Group chat already exists'),
     'item-not-found': _('Group chat does not exist'),
@@ -325,184 +394,6 @@ EME_MESSAGES = {
 }
 
 
-ACTIVITIES = {
-    'doing_chores': {
-        'category': _('Doing Chores'),
-        'buying_groceries': _('Buying Groceries'),
-        'cleaning': _('Cleaning'),
-        'cooking': _('Cooking'),
-        'doing_maintenance': _('Doing Maintenance'),
-        'doing_the_dishes': _('Doing the Dishes'),
-        'doing_the_laundry': _('Doing the Laundry'),
-        'gardening': _('Gardening'),
-        'running_an_errand': _('Running an Errand'),
-        'walking_the_dog': _('Walking the Dog')},
-    'drinking': {
-        'category': _('Drinking'),
-        'having_a_beer': _('Having a Beer'),
-        'having_coffee': _('Having Coffee'),
-        'having_tea': _('Having Tea')},
-    'eating': {
-        'category': _('Eating'),
-        'having_a_snack': _('Having a Snack'),
-        'having_breakfast': _('Having Breakfast'),
-        'having_dinner': _('Having Dinner'),
-        'having_lunch': _('Having Lunch')},
-    'exercising': {
-        'category': _('Exercising'),
-        'cycling': _('Cycling'),
-        'dancing': _('Dancing'),
-        'hiking': _('Hiking'),
-        'jogging': _('Jogging'),
-        'playing_sports': _('Playing Sports'),
-        'running': _('Running'),
-        'skiing': _('Skiing'),
-        'swimming': _('Swimming'),
-        'working_out': _('Working out')},
-    'grooming': {
-        'category': _('Grooming'),
-        'at_the_spa': _('At the Spa'),
-        'brushing_teeth': _('Brushing Teeth'),
-        'getting_a_haircut': _('Getting a Haircut'),
-        'shaving': _('Shaving'),
-        'taking_a_bath': _('Taking a Bath'),
-        'taking_a_shower': _('Taking a Shower')},
-    'having_appointment': {
-        'category': _('Having an Appointment')},
-    'inactive': {
-        'category': _('Inactive'),
-        'day_off': _('Day Off'),
-        'hanging_out': _('Hanging out'),
-        'hiding': _('Hiding'),
-        'on_vacation': _('On Vacation'),
-        'praying': _('Praying'),
-        'scheduled_holiday': _('Scheduled Holiday'),
-        'sleeping': _('Sleeping'),
-        'thinking': _('Thinking')},
-    'relaxing': {
-        'category': _('Relaxing'),
-        'fishing': _('Fishing'),
-        'gaming': _('Gaming'),
-        'going_out': _('Going out'),
-        'partying': _('Partying'),
-        'reading': _('Reading'),
-        'rehearsing': _('Rehearsing'),
-        'shopping': _('Shopping'),
-        'smoking': _('Smoking'),
-        'socializing': _('Socializing'),
-        'sunbathing': _('Sunbathing'),
-        'watching_tv': _('Watching TV'),
-        'watching_a_movie': _('Watching a Movie')},
-    'talking': {
-        'category': _('Talking'),
-        'in_real_life': _('In Real Life'),
-        'on_the_phone': _('On the Phone'),
-        'on_video_phone': _('On Video Phone')},
-    'traveling': {
-        'category': _('Traveling'),
-        'commuting': _('Commuting'),
-        'cycling': _('Cycling'),
-        'driving': _('Driving'),
-        'in_a_car': _('In a Car'),
-        'on_a_bus': _('On a Bus'),
-        'on_a_plane': _('On a Plane'),
-        'on_a_train': _('On a Train'),
-        'on_a_trip': _('On a Trip'),
-        'walking': _('Walking')},
-    'working': {
-        'category': _('Working'),
-        'coding': _('Coding'),
-        'in_a_meeting': _('In a Meeting'),
-        'studying': _('Studying'),
-        'writing': _('Writing')}}
-
-MOODS = {
-    'afraid': _('Afraid'),
-    'amazed': _('Amazed'),
-    'amorous': _('Amorous'),
-    'angry': _('Angry'),
-    'annoyed': _('Annoyed'),
-    'anxious': _('Anxious'),
-    'aroused': _('Aroused'),
-    'ashamed': _('Ashamed'),
-    'bored': _('Bored'),
-    'brave': _('Brave'),
-    'calm': _('Calm'),
-    'cautious': _('Cautious'),
-    'cold': _('Cold'),
-    'confident': _('Confident'),
-    'confused': _('Confused'),
-    'contemplative': _('Contemplative'),
-    'contented': _('Contented'),
-    'cranky': _('Cranky'),
-    'crazy': _('Crazy'),
-    'creative': _('Creative'),
-    'curious': _('Curious'),
-    'dejected': _('Dejected'),
-    'depressed': _('Depressed'),
-    'disappointed': _('Disappointed'),
-    'disgusted': _('Disgusted'),
-    'dismayed': _('Dismayed'),
-    'distracted': _('Distracted'),
-    'embarrassed': _('Embarrassed'),
-    'envious': _('Envious'),
-    'excited': _('Excited'),
-    'flirtatious': _('Flirtatious'),
-    'frustrated': _('Frustrated'),
-    'grateful': _('Grateful'),
-    'grieving': _('Grieving'),
-    'grumpy': _('Grumpy'),
-    'guilty': _('Guilty'),
-    'happy': _('Happy'),
-    'hopeful': _('Hopeful'),
-    'hot': _('Hot'),
-    'humbled': _('Humbled'),
-    'humiliated': _('Humiliated'),
-    'hungry': _('Hungry'),
-    'hurt': _('Hurt'),
-    'impressed': _('Impressed'),
-    'in_awe': _('In Awe'),
-    'in_love': _('In Love'),
-    'indignant': _('Indignant'),
-    'interested': _('Interested'),
-    'intoxicated': _('Intoxicated'),
-    'invincible': _('Invincible'),
-    'jealous': _('Jealous'),
-    'lonely': _('Lonely'),
-    'lost': _('Lost'),
-    'lucky': _('Lucky'),
-    'mean': _('Mean'),
-    'moody': _('Moody'),
-    'nervous': _('Nervous'),
-    'neutral': _('Neutral'),
-    'offended': _('Offended'),
-    'outraged': _('Outraged'),
-    'playful': _('Playful'),
-    'proud': _('Proud'),
-    'relaxed': _('Relaxed'),
-    'relieved': _('Relieved'),
-    'remorseful': _('Remorseful'),
-    'restless': _('Restless'),
-    'sad': _('Sad'),
-    'sarcastic': _('Sarcastic'),
-    'satisfied': _('Satisfied'),
-    'serious': _('Serious'),
-    'shocked': _('Shocked'),
-    'shy': _('Shy'),
-    'sick': _('Sick'),
-    'sleepy': _('Sleepy'),
-    'spontaneous': _('Spontaneous'),
-    'stressed': _('Stressed'),
-    'strong': _('Strong'),
-    'surprised': _('Surprised'),
-    'thankful': _('Thankful'),
-    'thirsty': _('Thirsty'),
-    'tired': _('Tired'),
-    'undefined': _('Undefined'),
-    'weak': _('Weak'),
-    'worried': _('Worried')
-}
-
 LOCATION_DATA = {
     'accuracy': _('accuracy'),
     'alt': _('alt'),
@@ -530,120 +421,129 @@ LOCATION_DATA = {
 
 
 SSLError = {
-    2: _("Unable to get issuer certificate"),
-    3: _("Unable to get certificate CRL"),
+    2: _('Unable to get issuer certificate'),
+    3: _('Unable to get certificate CRL'),
     4: _("Unable to decrypt certificate's signature"),
     5: _("Unable to decrypt CRL's signature"),
-    6: _("Unable to decode issuer public key"),
-    7: _("Certificate signature failure"),
-    8: _("CRL signature failure"),
-    9: _("Certificate is not yet valid"),
-    10: _("Certificate has expired"),
-    11: _("CRL is not yet valid"),
-    12: _("CRL has expired"),
+    6: _('Unable to decode issuer public key'),
+    7: _('Certificate signature failure'),
+    8: _('CRL signature failure'),
+    9: _('Certificate is not yet valid'),
+    10: _('Certificate has expired'),
+    11: _('CRL is not yet valid'),
+    12: _('CRL has expired'),
     13: _("Format error in certificate's notBefore field"),
     14: _("Format error in certificate's notAfter field"),
     15: _("Format error in CRL's lastUpdate field"),
     16: _("Format error in CRL's nextUpdate field"),
-    17: _("Out of memory"),
-    18: _("Self signed certificate"),
-    19: _("Self signed certificate in certificate chain"),
-    20: _("Unable to get local issuer certificate"),
-    21: _("Unable to verify the first certificate"),
-    22: _("Certificate chain too long"),
-    23: _("Certificate revoked"),
-    24: _("Invalid CA certificate"),
-    25: _("Path length constraint exceeded"),
-    26: _("Unsupported certificate purpose"),
-    27: _("Certificate not trusted"),
-    28: _("Certificate rejected"),
-    29: _("Subject issuer mismatch"),
-    30: _("Authority and subject key identifier mismatch"),
-    31: _("Authority and issuer serial number mismatch"),
-    32: _("Key usage does not include certificate signing"),
-    50: _("Application verification failure"),
+    17: _('Out of memory'),
+    18: _('Self signed certificate'),
+    19: _('Self signed certificate in certificate chain'),
+    20: _('Unable to get local issuer certificate'),
+    21: _('Unable to verify the first certificate'),
+    22: _('Certificate chain too long'),
+    23: _('Certificate revoked'),
+    24: _('Invalid CA certificate'),
+    25: _('Path length constraint exceeded'),
+    26: _('Unsupported certificate purpose'),
+    27: _('Certificate not trusted'),
+    28: _('Certificate rejected'),
+    29: _('Subject issuer mismatch'),
+    30: _('Authority and subject key identifier mismatch'),
+    31: _('Authority and issuer serial number mismatch'),
+    32: _('Key usage does not include certificate signing'),
+    50: _('Application verification failure'),
 }
 
 
-THANKS = u"""\
-Alexander Futász
-Alexander V. Butenko
-Alexey Nezhdanov
-Alfredo Junix
-Anaël Verrier
-Anders Ström
-Andrew Sayman
-Anton Shmigirilov
-Christian Bjälevik
-Christophe Got
-Christoph Neuroth
-David Campey
-Dennis Craven
-Fabian Neumann
-Filippos Papadopoulos
-Francisco Alburquerque Parra (Membris Khan)
-Frederic Lory
-Fridtjof Bussefor
-Geobert Quach
-Guillaume Morin
-Gustavo J. A. M. Carneiro
-Ivo Anjo
-Josef Vybíral
-Juraj Michalek
-Kjell Braden
-Luis Peralta
-Michael Scherer
-Michele Campeotto
-Mike Albon
-Miguel Fonseca
-Norman Rasmussen
-Oscar Hellström
-Peter Saint-Andre
-Petr Menšík
-Sergey Kuleshov
-Stavros Giannouris
-Stian B. Barmen
-Thilo Molitor
-Thomas Klein-Hitpaß
-Urtzi Alfaro
-Witold Kieraś
-Yakov Bezrukov
-Yavor Doganov
-""".strip().split("\n")
+VOWELS = 'aeiou'
 
-ARTISTS = u"""\
-Anders Ström
-Christophe Got
-Dennis Craven
-Dmitry Korzhevin
-Guillaume Morin
-Gvorcek Spajreh
-Josef Vybíral
-Membris Khan
-Rederick Asher
-Jakub Szypulka
-""".strip().split("\n")
 
-DEVS_CURRENT = u"""\
-Yann Leboulanger (asterix AT lagaule.org)
-Philipp Hörist (philipp AT hoerist.com)
-Daniel Brötzmann (wurstsalat AT posteo.de)
-André Apitzsch
-""".strip().split("\n")
+CONSONANTS = 'bcdfghjklmnpqrstvwxyz'
 
-DEVS_PAST = u"""\
-Stefan Bethge (stefan AT lanpartei.de)
-Alexander Cherniuk (ts33kr AT gmail.com)
-Stephan Erb (steve-e AT h3c.de)
-Vincent Hanquez (tab AT snarc.org)
-Dimitur Kirov (dkirov AT gmail.com)
-Nikos Kouremenos (kourem AT gmail.com)
-Julien Pivotto (roidelapluie AT gmail.com)
-Jonathan Schleifer (js-gajim AT webkeks.org)
-Travis Shirk (travis AT pobox.com)
-Brendan Taylor (whateley AT gmail.com)
-Jean-Marie Traissard (jim AT lapin.org)
-""".strip().split("\n")
+
+THANKS = [
+    'Alexander Futász',
+    'Alexander V. Butenko',
+    'Alexey Nezhdanov',
+    'Alfredo Junix',
+    'Anaël Verrier',
+    'Anders Ström',
+    'Andrew Sayman',
+    'Anton Shmigirilov',
+    'Christian Bjälevik',
+    'Christophe Got',
+    'Christoph Neuroth',
+    'David Campey',
+    'Dennis Craven',
+    'Fabian Neumann',
+    'Filippos Papadopoulos',
+    'Francisco Alburquerque Parra (Membris Khan)',
+    'Frederic Lory',
+    'Fridtjof Bussefor',
+    'Geobert Quach',
+    'Guillaume Morin',
+    'Gustavo J. A. M. Carneiro',
+    'Ivo Anjo',
+    'Josef Vybíral',
+    'Juraj Michalek',
+    'Kjell Braden',
+    'Luis Peralta',
+    'Michael Scherer',
+    'Michele Campeotto',
+    'Mike Albon',
+    'Miguel Fonseca',
+    'Norman Rasmussen',
+    'Oscar Hellström',
+    'Peter Saint-Andre',
+    'Petr Menšík',
+    'Sergey Kuleshov',
+    'Stavros Giannouris',
+    'Stian B. Barmen',
+    'Thilo Molitor',
+    'Thomas Klein-Hitpaß',
+    'Urtzi Alfaro',
+    'Witold Kieraś',
+    'Yakov Bezrukov',
+    'Yavor Doganov',
+]
+
+
+ARTISTS = [
+    'Anders Ström',
+    'Christophe Got',
+    'Dennis Craven',
+    'Dmitry Korzhevin',
+    'Guillaume Morin',
+    'Gvorcek Spajreh',
+    'Josef Vybíral',
+    'Membris Khan',
+    'Rederick Asher',
+    'Jakub Szypulka',
+]
+
+
+DEVS_CURRENT = [
+    'Yann Leboulanger (asterix AT lagaule.org)',
+    'Philipp Hörist (philipp AT hoerist.com)',
+    'Daniel Brötzmann (wurstsalat AT posteo.de)',
+    'André Apitzsch',
+]
+
+
+DEVS_PAST = [
+    'Stefan Bethge (stefan AT lanpartei.de)',
+    'Alexander Cherniuk (ts33kr AT gmail.com)',
+    'Stephan Erb (steve-e AT h3c.de)',
+    'Vincent Hanquez (tab AT snarc.org)',
+    'Dimitur Kirov (dkirov AT gmail.com)',
+    'Nikos Kouremenos (kourem AT gmail.com)',
+    'Julien Pivotto (roidelapluie AT gmail.com)',
+    'Jonathan Schleifer (js-gajim AT webkeks.org)',
+    'Travis Shirk (travis AT pobox.com)',
+    'Brendan Taylor (whateley AT gmail.com)',
+    'Jean-Marie Traissard (jim AT lapin.org)',
+]
 
 
 RFC5646_LANGUAGE_TAGS = {
@@ -868,7 +768,7 @@ RFC5646_LANGUAGE_TAGS = {
     'uz-UZ': 'Uzbek (Latin) (Uzbekistan)',
     'uz-Cyrl-UZ': 'Uzbek (Cyrillic) (Uzbekistan)',
     'vi': 'Vietnamese',
-    'vi-VN': 'Vietnamese (Viet Nam)',
+    'vi-VN': 'Vietnamese (Vietnam)',
     'xh': 'Xhosa',
     'xh-ZA': 'Xhosa (South Africa)',
     'zh': 'Chinese',
@@ -881,20 +781,25 @@ RFC5646_LANGUAGE_TAGS = {
     'zu-ZA': 'Zulu (South Africa)'
 }
 
-# pylint: disable=line-too-long
+
 GIO_TLS_ERRORS = {
-    Gio.TlsCertificateFlags.UNKNOWN_CA: _('The signing certificate authority is not known'),
-    Gio.TlsCertificateFlags.REVOKED: _('The certificate has been revoked'),
-    Gio.TlsCertificateFlags.BAD_IDENTITY: _('The certificate does not match the expected identity of the site'),
-    Gio.TlsCertificateFlags.INSECURE: _('The certificate’s algorithm is insecure'),
-    Gio.TlsCertificateFlags.NOT_ACTIVATED: _('The certificate’s activation time is in the future'),
+    Gio.TlsCertificateFlags.UNKNOWN_CA: _(
+        'The signing certificate authority is not known'),
+    Gio.TlsCertificateFlags.REVOKED: _(
+        'The certificate has been revoked'),
+    Gio.TlsCertificateFlags.BAD_IDENTITY: _(
+        'The certificate does not match the expected identity of the site'),
+    Gio.TlsCertificateFlags.INSECURE: _(
+        'The certificate’s algorithm is insecure'),
+    Gio.TlsCertificateFlags.NOT_ACTIVATED: _(
+        'The certificate’s activation time is in the future'),
     Gio.TlsCertificateFlags.GENERIC_ERROR: _('Unknown validation error'),
     Gio.TlsCertificateFlags.EXPIRED: _('The certificate has expired'),
 }
-# pylint: enable=line-too-long
 
 
 class FTState(Enum):
+    INIT = 'init'
     PREPARING = 'prepare'
     ENCRYPTING = 'encrypting'
     DECRYPTING = 'decrypting'
@@ -902,6 +807,7 @@ class FTState(Enum):
     IN_PROGRESS = 'progress'
     FINISHED = 'finished'
     ERROR = 'error'
+    CANCELLED = 'cancelled'
 
     @property
     def is_preparing(self):
@@ -931,6 +837,16 @@ class FTState(Enum):
     def is_error(self):
         return self == FTState.ERROR
 
+    @property
+    def is_cancelled(self):
+        return self == FTState.CANCELLED
+
+    @property
+    def is_active(self):
+        return not (self.is_error or
+                    self.is_cancelled or
+                    self.is_finished)
+
 
 SASL_ERRORS = {
     'aborted': _('Authentication aborted'),
@@ -964,6 +880,7 @@ COMMON_FEATURES = [
     Namespace.SECLABEL,
     Namespace.CONFERENCE,
     Namespace.CORRECT,
+    Namespace.CHATMARKERS,
     Namespace.EME,
     Namespace.XHTML_IM,
     Namespace.HASHES_2,
@@ -981,16 +898,297 @@ COMMON_FEATURES = [
     Namespace.JINGLE_BYTESTREAM,
     Namespace.JINGLE_IBB,
     Namespace.AVATAR_METADATA + '+notify',
+    Namespace.MESSAGE_MODERATE
 ]
 
 
 SHOW_LIST = [
     'offline',
     'connecting',
-    'online',
-    'chat',
     'away',
     'xa',
+    'chat',
+    'online',
     'dnd',
-    'error'
 ]
+
+SHOW_STRING = {
+    'dnd': _('Busy'),
+    'xa': _('Not Available'),
+    'chat': _('Free for Chat'),
+    'online': Q_('?user status:Available'),
+    'connecting': _('Connecting'),
+    'away': _('Away'),
+    'offline': _('Offline'),
+}
+
+SHOW_STRING_MNEMONIC = {
+    'dnd': _('_Busy'),
+    'xa': _('_Not Available'),
+    'chat': _('_Free for Chat'),
+    'online': Q_('?user status:_Available'),
+    'connecting': _('Connecting'),
+    'away': _('A_way'),
+    'offline': _('_Offline'),
+}
+
+GAJIM_FAQ_URI = 'https://dev.gajim.org/gajim/gajim/wikis/help/gajimfaq'
+GAJIM_WIKI_URI = 'https://dev.gajim.org/gajim/gajim/wikis'
+GAJIM_SUPPORT_JID = 'gajim@conference.gajim.org'
+
+URI_SCHEMES = {
+    'aaa://',
+    'aaas://',
+    'acap://',
+    'cap://',
+    'cid:',
+    'crid://',
+    'data:',
+    'dav:',
+    'dict://',
+    'dns:',
+    'fax:',
+    'file:/',
+    'ftp://',
+    'geo:',
+    'go:',
+    'gopher://',
+    'h323:',
+    'http://',
+    'https://',
+    'iax:',
+    'icap://',
+    'im:',
+    'imap://',
+    'info:',
+    'ipp://',
+    'iris:',
+    'iris.beep:',
+    'iris.xpc:',
+    'iris.xpcs:',
+    'iris.lwz:',
+    'ldap://',
+    'mailto:',
+    'mid:',
+    'modem:',
+    'msrp://',
+    'msrps://',
+    'mtqp://',
+    'mupdate://',
+    'news:',
+    'nfs://',
+    'nntp://',
+    'opaquelocktoken:',
+    'pop://',
+    'pres:',
+    'prospero://',
+    'rtsp://',
+    'service:',
+    'sip:',
+    'sips:',
+    'sms:',
+    'snmp://',
+    'soap.beep://',
+    'soap.beeps://',
+    'tag:',
+    'tel:',
+    'telnet://',
+    'tftp://',
+    'thismessage:/',
+    'tip://',
+    'tv:',
+    'urn://',
+    'vemmi://',
+    'xmlrpc.beep://',
+    'xmlrpc.beeps://',
+    'xmpp:',
+    'z39.50r://',
+    'z39.50s://',
+    'about:',
+    'apt:',
+    'cvs://',
+    'daap://',
+    'ed2k://',
+    'feed:',
+    'fish://',
+    'git://',
+    'iax2:',
+    'irc://',
+    'ircs://',
+    'ldaps://',
+    'magnet:',
+    'mms://',
+    'rsync://',
+    'ssh://',
+    'svn://',
+    'sftp://',
+    'smb://',
+    'webcal://',
+    'aesgcm://',
+}
+
+
+# This is an excerpt of Media Types from
+# https://www.iana.org/assignments/media-types/media-types.xhtml
+# plus some additions
+MIME_TYPES = (
+    # application/
+    'application/calendar+json',
+    'application/calendar+xml',
+    'application/epub+zip',
+    'application/json',
+    'application/mp4',
+    'application/msword',
+    'application/octet-stream',
+    'application/ogg',
+    'application/pdf',
+    'application/pgp-encrypted',
+    'application/pgp-signature',
+    'application/postscript',
+    'application/rtf',
+    'application/vcard+json',
+    'application/vcard+xml',
+    'application/vnd.amazon.mobi8-ebook',
+    'application/vnd.google-earth.kml+xml',
+    'application/vnd.google-earth.kmz',
+    # Start office
+    'application/vnd.ms-access',
+    'application/vnd.ms-excel',
+    'application/vnd.ms-excel.addin.macroEnabled.12',
+    'application/vnd.ms-excel.sheet.binary.macroEnabled.12',
+    'application/vnd.ms-excel.sheet.macroEnabled.12',
+    'application/vnd.ms-excel.template.macroEnabled.12',
+    'application/vnd.ms-powerpoint',
+    'application/vnd.ms-powerpoint.addin.macroEnabled.12',
+    'application/vnd.ms-powerpoint.presentation.macroEnabled.12',
+    'application/vnd.ms-powerpoint.slideshow.macroEnabled.12',
+    'application/vnd.ms-powerpoint.template.macroEnabled.12',
+    'application/vnd.ms-word.document.macroEnabled.12',
+    'application/vnd.ms-word.template.macroEnabled.12',
+    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
+    'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
+    'application/vnd.openxmlformats-officedocument.presentationml.template',
+    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
+    'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
+    'application/vnd.openxmlformats-officedocument.vmlDrawing',
+    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
+    'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
+    # End office
+    'application/vnd.sqlite3',
+    'application/zip',
+    # audio/*
+    'audio/aac',
+    'audio/ac3',
+    'audio/flac',
+    'audio/mp4',
+    'audio/mpeg',
+    'audio/ogg',
+    'audio/opus',
+    'audio/wav',
+    'audio/x-flac',
+    'audio/x-m4a',
+    'audio/x-matroska',
+    # font/*
+    'font/ttf',
+    'font/woff',
+    'font/woff2',
+    # image/*
+    'image/webp',
+    'image/avif',
+    'image/jxl',
+    'image/bmp',
+    'image/x-bmp',
+    'image/x-ms-bmp',
+    'image/gif',
+    'image/heic',
+    'image/heif',
+    'image/jpeg',
+    'image/png',
+    'image/svg+xml',
+    'image/tiff',
+    'image/vnd.adobe.photoshop',
+    'image/vnd.dwg',
+    'image/vnd.dxf',
+    'image/vnd.microsoft.icon',
+    'image/x-icon',
+    'image/x-xcf',
+    # model/*
+    'model/mtl',
+    'model/obj',
+    'model/stl',
+    # text/*
+    'text/calendar',
+    'text/csv',
+    'text/markdown',
+    'text/rtf',
+    'text/vcard',
+    # video/*
+    'video/H264',
+    'video/H265',
+    'video/mp4',
+    'video/mpeg4-generic',
+    'video/ogg',
+    'video/quicktime',
+    'video/vc1',
+    'video/VP8',
+    'video/webm',
+    'video/x-matroska',
+    'video/x-msvideo',
+)
+
+
+TRUST_SYMBOL_DATA = {
+    Trust.UNTRUSTED: ('dialog-error-symbolic',
+                      _('Untrusted'),
+                      'error-color'),
+    Trust.UNDECIDED: ('security-low-symbolic',
+                      _('Trust Not Decided'),
+                      'warning-color'),
+    Trust.BLIND: ('security-medium-symbolic',
+                  _('Unverified'),
+                  'encrypted-color'),
+    Trust.VERIFIED: ('security-high-symbolic',
+                     _('Verified'),
+                     'encrypted-color')
+}
+
+
+THRESHOLD_OPTIONS = {
+    -1: _('No Sync'),
+    1: _('1 Day'),
+    2: _('2 Days'),
+    7: _('1 Week'),
+    30: _('1 Month'),
+    0: _('No Threshold'),
+}
+
+
+@total_ordering
+class PresenceShowExt(Enum):
+
+    '''
+    This extends nbxmpp.const.PresenceShow for convenience
+    with an OFFLINE member
+    '''
+
+    OFFLINE = 'offline'
+
+    @property
+    def is_offline(self):
+        return self == PresenceShowExt.OFFLINE
+
+    def __lt__(self, other: Union[PresenceShowExt, PresenceShow]) -> bool:
+        if isinstance(other, PresenceShowExt):
+            return False
+        if not isinstance(other, PresenceShow):  # pyright: ignore
+            return NotImplemented
+        return True
+
+
+# Replace keys with Gio.TlsProtocolVersion enum once
+# we require GLib >= 2.69
+TLS_VERSION_STRINGS = {
+    3: '1.1',
+    4: '1.2',
+    5: '1.3',
+}
